import _extends from "@babel/runtime/helpers/esm/extends";
import { stringify } from 'qs';
import { writeData2File } from '@pactflow/pact-msw-adapter/dist/utils/utils.js';
var flatEntries = function flatEntries(input, prefix) {
  if (prefix === void 0) {
    prefix = '';
  }
  return Object.entries(input).flatMap(function (_ref) {
    var _ref2;
    var key = _ref[0],
      value = _ref[1];
    var isObject = typeof value === 'object';
    return isObject ? flatEntries(value, "" + prefix + key + ".") : (_ref2 = {}, _ref2["" + prefix + key] = {
      match: 'type'
    }, _ref2);
  });
};
var reduceArrayToObject = function reduceArrayToObject(items) {
  return items.reduce(function (objAcc, item) {
    return _extends({}, objAcc, item);
  }, {});
};
var getMatchingRules = function getMatchingRules(body) {
  var matchingRulesArray = flatEntries(body, '$.body.');
  var matchingRules = reduceArrayToObject(matchingRulesArray);
  return matchingRules;
};
export var writerFactory = function writerFactory(providers) {
  return function (path, data) {
    var interactions = data.interactions,
      provider = data.provider;
    var name = provider.name;
    data.interactions = interactions.map(function (interaction) {
      var request = interaction.request;
      var path = request.path,
        query = request.query;
      var definedInteraction = providers[name].find(function (_ref3) {
        var endpoint = _ref3.endpoint,
          definedQuery = _ref3.query;
        var matchedEndpoint = endpoint === path;
        if (query) {
          return matchedEndpoint && query === toQueryString(definedQuery);
        }
        return matchedEndpoint;
      });
      var description = definedInteraction.description,
        providerState = definedInteraction.state,
        addMatchingRules = definedInteraction.addMatchingRules;
      return _extends({}, interaction, {
        response: _extends({}, interaction.response, addMatchingRules && {
          matchingRules: getMatchingRules(interaction.response.body)
        }),
        description: description,
        providerState: providerState
      });
    });
    console.log("Writing the Pact file \"" + path + "\""); // eslint-disable-line
    writeData2File(path, data);
  };
};
export var mapProviders = function mapProviders(providers) {
  return Object.keys(providers).reduce(function (reducedProviders, key) {
    var _extends2;
    var endpoints = providers[key].map(function (_ref4) {
      var endpoint = _ref4.endpoint,
        query = _ref4.query;
      return query ? endpoint + "?" + toQueryString(query) : endpoint;
    });
    return _extends({}, reducedProviders, (_extends2 = {}, _extends2[key] = endpoints, _extends2));
  }, {});
};
export var getContractTests = function getContractTests(_ref5) {
  var apiUrl = _ref5.apiUrl,
    providers = _ref5.providers;
  return Object.keys(providers).reduce(function (tests, key) {
    return [].concat(tests, providers[key]);
  }, []);
};
export var toQueryString = function toQueryString(queryParams, options) {
  if (options === void 0) {
    options = {};
  }
  var _options = options,
    arrayFormat = _options.arrayFormat,
    delimiter = _options.delimiter,
    _options$encode = _options.encode,
    encode = _options$encode === void 0 ? true : _options$encode;
  var mergedOptions = _extends({
    encode: encode
  }, typeof arrayFormat !== 'undefined' && {
    arrayFormat: arrayFormat
  }, typeof delimiter !== 'undefined' && {
    delimiter: delimiter
  });
  return stringify(queryParams, mergedOptions);
};